/* 
 * Formation ENVOL 2014 - Atelier électronique embarquée 2
 * Station météo 3.0 - Datalogging toutes les 10mn + Affichage live sur LCD
 * Matériel : idem + Mémoire 2.0 (DS1307) + Deuligne
 */

    /* RELEVÉ DONNÉES MÉTÉO */
    
// Inclusion de la librairie nécessaire pour la lecture du DHT11
#include <DHT.h>

//Déclaration de la pin reliée au capteur
#define DHTPIN 2

// Déclaration du type de capteur utilisé et initialisation
#define DHTTYPE DHT11
DHT dht(DHTPIN, DHTTYPE);

// Déclaration des variables nécessaires
// Cette fois les variables du DHT11 sont déclarées en int pour tronquer la partie décimale
// 1. pour gagner de la place sur l'affichage
// 2. parce que la précision du capteur n'est de toute façon pas suffisante !
int hum;
int temp;
int lum;

    /* DATALOGGING */

// Inclusion des librairies nécessaires à l'utilisation du shield Mémoire
// (les deux premières sont intégrées à l'IDE Arduino)
#include <SD.h>
#include <Wire.h>
#include <RTClib.h>

// Déclaration des objets liés à l'horloge
RTC_DS1307 RTC;
DateTime timestamp;
int updateTime;

    /* AFFICHAGE LCD */

// Inclusion des librairies nécessaires à l'utilisation du shield Deuligne
// Inutile d'inclure la librairie Wire.h à nouveau !
#include <Deuligne.h>

// Déclaration de l'objet LCD
Deuligne LCD;

void setup()
{
  // Ici plus de liaison série, on a fini le debug 
  // On envoie les données directement sur les éléments du montage :)
  
  // Initialisation DHT11
  dht.begin();
  
  // Initialisation Deuligne
  LCD.init();
  
  // Initialisation carte SD
  pinMode(10, OUTPUT);
  if (!SD.begin(10))
  {
    LCD.print("Card failed");
    return;
  }
  
  // Initialisation RTC
  Wire.begin();
  RTC.begin();
  
  // Mise à l'heure suivant paramètres système
  RTC.adjust(DateTime(__DATE__, __TIME__));
  if (!RTC.isrunning())
  {
    LCD.print("RTC failed");
    return;
  }
  
  // Sauvegarde de l'heure de départ pour commencer l'enregistrement des données
  DateTime startTime = RTC.now();
  updateTime = startTime.second();
  
  // Mise en page du tableau CSV
  File fichier = SD.open("meteo.csv", FILE_WRITE);
  fichier.print("Date;Heure;Humidite (%);Temperature (*C);Luminosite (%)\n");
  fichier.close();
}

void loop()
{
  // Relevé des valeurs en provenance du DHT11
  hum = dht.readHumidity();
  temp = dht.readTemperature();
  
  // Relevé des valeurs en provenance de la photorésistance et conversion en pourcentage
  // Si Deuligne penser à changer la pin de la photorésistance
  // pour éviter le conflit avec le joystick ! > A1
  lum = analogRead(1);
  lum = map(lum, 150, 1000, 0, 100); // Valeurs à rajuster si besoin
  
  // Relevé de l'heure
  timestamp = RTC.now();
  
  // Affichage sur écran LCD
  // De l'heure
  LCD.print(timestamp.hour(), DEC);
  LCD.print(':');
  LCD.print(timestamp.minute(), DEC);
  LCD.print(':');
  // Impression du '0' quand il s'est écoulé moins de 10 secondes
  if (timestamp.second() < 10)
  {
    LCD.print('0');
  }
  LCD.print(timestamp.second(), DEC);
  
  // Puis des conditions météo
  LCD.print(" H= ");
  LCD.print(hum);
  LCD.setCursor(0, 1); // Placement du curseur sur la 1e colonne de la 2e ligne
  LCD.print("T= ");
  LCD.print(temp);
  LCD.print(" L= ");
  LCD.print(lum);
  
  // On regarde s'il est temps de faire un relevé sur la carte SD
  // C'est-à-dire s'il s'est écoulé plus de 10 secondes depuis le dernier relevé
  if ((timestamp.second() - 10) == updateTime)
  {
    // auquel cas on remet à jour la variable concernée
    updateTime = timestamp.second();
    printFile();
  }
  // cas particulier : lorsque le dernier relevé est à __:__:5_, le prochain doit se faire à __:__:0_
  else if ((timestamp.second() < 10) && ((timestamp.second() + 50) == updateTime))
  {
    updateTime = timestamp.second();
    printFile();
  }
  
  delay(1000);
  LCD.clear(); // Ecran vierge pour la prochaine boucle
}

void printFile()
{
  // Impression dans fichier texte sur carte SD
  File fichier = SD.open("meteo.csv", FILE_WRITE);
  
  // Date et heure
  fichier.print(timestamp.year(), DEC);
  fichier.print('/');
  fichier.print(timestamp.month(), DEC);
  fichier.print('/');
  fichier.print(timestamp.day(), DEC);
  fichier.print(';');
  fichier.print(timestamp.hour(), DEC);
  fichier.print(':');
  fichier.print(timestamp.minute(), DEC);
  fichier.print(':');
  // Impression du '0' quand il s'est écoulé moins de 10 secondes
  if (timestamp.second() < 10)
  {
    fichier.print('0');
  }
  fichier.print(timestamp.second(), DEC);
  fichier.print(";");
  
  // COnditions météo
  fichier.print(hum);
  fichier.print(";");
  fichier.print(temp);
  fichier.print(";");
  fichier.print(lum);
  fichier.println("");
  fichier.close();
}
